@@ -1,16 +1,15 @@
 package org.jmx4perl.config;
 
+import java.io.*;
+import java.lang.management.ManagementFactory;
+import java.util.Set;
+
+import javax.management.*;
+
 import org.jmx4perl.backend.MBeanServerHandler;
 import org.jmx4perl.history.HistoryKey;
 import org.jmx4perl.history.HistoryStore;
 
-import javax.management.*;
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
-import java.io.ObjectOutputStream;
-import java.lang.management.ManagementFactory;
-import java.util.Set;
-
 
 /*
  * jmx4perl - WAR Agent for exporting JMX via JSON
@@ -39,7 +38,7 @@ import java.util.Set;
  * @author roland
  * @since Jun 12, 2009
  */
-public class Config implements ConfigMBean {
+public class Config implements ConfigMBean,MBeanRegistration {
 
     private HistoryStore historyStore;
     private DebugStore debugStore;
@@ -54,12 +53,13 @@ public class Config implements ConfigMBean {
         mBeanServerHandler = pMBeanServerHandler;
     }
 
-    public void setHistoryEntriesForAttribute(String pMBean, String pAttribute, String pPath, String pTarget, int pMaxEntries) {
+    public void setHistoryEntriesForAttribute(String pMBean, String pAttribute, String pPath, String pTarget, int pMaxEntries)
+            throws MalformedObjectNameException {
         HistoryKey key = new HistoryKey(pMBean,pAttribute,pPath,pTarget);
         historyStore.configure(key,pMaxEntries);
     }
 
-    public void setHistoryEntriesForOperation(String pMBean, String pOperation, String pTarget, int pMaxEntries) {
+    public void setHistoryEntriesForOperation(String pMBean, String pOperation, String pTarget, int pMaxEntries) throws MalformedObjectNameException {
         HistoryKey key = new HistoryKey(pMBean,pOperation,pTarget);
         historyStore.configure(key,pMaxEntries);
     }
@@ -155,4 +155,20 @@ public class Config implements ConfigMBean {
         oOut.writeObject(historyStore);
         return bOut.size();
     }
+
+    // ========================================================================
+
+    // Provide our own name on registration
+    public ObjectName preRegister(MBeanServer server, ObjectName name) throws Exception {
+        return new ObjectName(OBJECT_NAME);
+    }
+
+    public void postRegister(Boolean registrationDone) {
+    }
+
+    public void preDeregister() throws Exception {
+    }
+
+    public void postDeregister() {
+    }
 }
