@@ -37,8 +37,7 @@ import java.util.Set;
 public abstract class JsonRequestHandler {
 
     // Restrictor for restricting operations
-
-    private final Restrictor restrictor;
+    protected final Restrictor restrictor;
 
     protected JsonRequestHandler(Restrictor pRestrictor) {
         restrictor = pRestrictor;
@@ -132,23 +131,4 @@ public abstract class JsonRequestHandler {
                 throws InstanceNotFoundException, AttributeNotFoundException, ReflectionException, MBeanException, IOException {
         return null;
     }
-
-    /**
-     * Use the path for the return value by default
-     *
-     * @return true
-     */
-    public boolean useReturnValueWithPath() {
-        return true;
-    }
-
-    /**
-     * Get the restrictor which is currently active
-     *
-     * @return restrictor
-     */
-    protected Restrictor getRestrictor() {
-        return restrictor;
-    }
-
 }
