@@ -1,16 +1,18 @@
 package org.jmx4perl.history;
 
-import java.io.Serializable;
-import java.util.*;
+import org.json.simple.JSONArray;
+import org.json.simple.JSONObject;
+
+import org.jmx4perl.JmxRequest;
 
 import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
 
-import org.jmx4perl.JmxRequest;
-import org.json.simple.JSONObject;
-
 import static org.jmx4perl.JmxRequest.Type.*;
 
+import java.util.*;
+import java.io.Serializable;
+
 /*
  * jmx4perl - WAR Agent for exporting JMX via JSON
  *
@@ -50,37 +52,17 @@ public class HistoryStore implements Serializable {
     private Map<HistoryKey, HistoryEntry> historyStore;
     private Map<HistoryKey, Integer /* max entries */> patterns;
 
-    // Keys used in JSON representation
-    private static final String KEY_HISTORY = "history";
-    private static final String KEY_VALUE = "value";
-    private static final String KEY_TIMESTAMP = "timestamp";
-
-    /**
-     * Constructor for a history store
-     *
-     * @param pTotalMaxEntries number of entries to hold at max. Even when configured, this maximum can not
-     *        be overwritten. This is a hard limit.
-     */
     public HistoryStore(int pTotalMaxEntries) {
         globalMaxEntries = pTotalMaxEntries;
         historyStore = new HashMap<HistoryKey, HistoryEntry>();
         patterns = new HashMap<HistoryKey, Integer>();
     }
 
-    /**
-     * Get the number
-     * @return
-     */
-    public synchronized int getGlobalMaxEntries() {
+    public int getGlobalMaxEntries() {
         return globalMaxEntries;
     }
 
-    /**
-     * Set the global maximum limit for history entries
-     *
-     * @param pGlobalMaxEntries limit
-     */
-    public synchronized void setGlobalMaxEntries(int pGlobalMaxEntries) {
+    synchronized public void setGlobalMaxEntries(int pGlobalMaxEntries) {
         globalMaxEntries = pGlobalMaxEntries;
         // Refresh all entries
         for (HistoryEntry entry : historyStore.values()) {
@@ -97,7 +79,7 @@ public class HistoryStore implements Serializable {
      * @param pMaxEntries number of maximal entries. If larger than globalMaxEntries,
      * then globalMaxEntries is used instead.
      */
-    public synchronized void configure(HistoryKey pKey,int pMaxEntries) {
+    public void configure(HistoryKey pKey,int pMaxEntries) {
         int maxEntries = pMaxEntries > globalMaxEntries ? globalMaxEntries : pMaxEntries;
 
         // Remove entries if set to 0
@@ -127,47 +109,6 @@ public class HistoryStore implements Serializable {
         }
     }
 
-    /**
-     * Reset the complete store.
-     */
-    public synchronized void reset() {
-        historyStore = new HashMap<HistoryKey, HistoryEntry>();
-        patterns = new HashMap<HistoryKey, Integer>();
-    }
-
-    /**
-     * Update the history store with the value of an an read, write or execute operation. Also, the timestamp
-     * of the insertion is recorded. Also, the recorded history values are added to the given json value.
-     *
-     * @param pJmxReq request for which an entry should be added in this history store
-     * @param pJson the JSONObject to which to add the history.
-     */
-    public synchronized void updateAndAdd(JmxRequest pJmxReq, JSONObject pJson) {
-        long timestamp = System.currentTimeMillis() / 1000;
-        pJson.put(KEY_TIMESTAMP,timestamp);
-
-        JmxRequest.Type type  = pJmxReq.getType();
-        if (type == EXEC || type == WRITE) {
-            HistoryEntry entry = historyStore.get(new HistoryKey(pJmxReq));
-            if (entry != null) {
-                synchronized(entry) {
-                    // A history data to json object for the response
-                    pJson.put(KEY_HISTORY,entry.jsonifyValues());
-
-                    // Update history for next time
-                    if (type == EXEC) {
-                        entry.add(pJson.get(KEY_VALUE),timestamp);
-                    } else if (type == WRITE) {
-                        // The new value to set as string representation
-                        entry.add(pJmxReq.getValue(),timestamp);
-                    }
-                }
-            }
-        } else if (type == READ) {
-            updateReadHistory(pJmxReq, pJson, timestamp);
-        }
-    }
-
     // Remove entries
     private void removeEntries(HistoryKey pKey) {
         if (pKey.isMBeanPattern()) {
@@ -190,6 +131,40 @@ public class HistoryStore implements Serializable {
         }
     }
 
+    /**
+     * Reset the complete store
+     */
+    public synchronized void reset() {
+        historyStore = new HashMap<HistoryKey, HistoryEntry>();
+        patterns = new HashMap<HistoryKey, Integer>();
+    }
+
+    public synchronized void updateAndAdd(JmxRequest pJmxReq, JSONObject pJson) {
+        long timestamp = System.currentTimeMillis() / 1000;
+        pJson.put("timestamp",timestamp);
+
+        JmxRequest.Type type  = pJmxReq.getType();
+        if (type == EXEC || type == WRITE) {
+            HistoryEntry entry = historyStore.get(new HistoryKey(pJmxReq));
+            if (entry != null) {
+                synchronized(entry) {
+                    // A history data to json object for the response
+                    pJson.put("history",entry.jsonifyValues());
+
+                    // Update history for next time
+                    if (type == EXEC) {
+                        entry.add(pJson.get("value"),timestamp);
+                    } else if (type == WRITE) {
+                        // The new value to set as string representation
+                        entry.add(pJmxReq.getValue(),timestamp);
+                    }
+                }
+            }
+        } else if (type == READ) {
+            updateReadHistory(pJmxReq, pJson, timestamp);
+        }
+    }
+
     // Update potentially multiple history entries for a READ request which could
     // return multiple values with a single request
     private void updateReadHistory(JmxRequest pJmxReq, JSONObject pJson, long pTimestamp)  {
@@ -198,7 +173,7 @@ public class HistoryStore implements Serializable {
             // We have a pattern and hence a value structure
             // of bean -> attribute_key -> attribute_value
             JSONObject history = new JSONObject();
-            for (Map.Entry<String,Object> beanEntry : ((Map<String,Object>) pJson.get(KEY_VALUE)).entrySet()) {
+            for (Map.Entry<String,Object> beanEntry : ((Map<String,Object>) pJson.get("value")).entrySet()) {
                 String beanName = beanEntry.getKey();
                 JSONObject beanHistory =
                         addAttributesFromComplexValue(
@@ -211,7 +186,7 @@ public class HistoryStore implements Serializable {
                 }
             }
             if (history.size() > 0) {
-                pJson.put(KEY_HISTORY,history);
+                pJson.put("history",history);
             }
         } else if (!pJmxReq.isSingleAttribute() || pJmxReq.getAttributeName() == null) {
             // Multiple attributes, but a single bean.
@@ -219,19 +194,19 @@ public class HistoryStore implements Serializable {
             // attribute_key -> attribute_value
             JSONObject history = addAttributesFromComplexValue(
                     pJmxReq,
-                    ((Map<String,Object>) pJson.get(KEY_VALUE)),
+                    ((Map<String,Object>) pJson.get("value")),
                     pJmxReq.getObjectNameAsString(),
                     pTimestamp);
             if (history.size() > 0) {
-                pJson.put(KEY_HISTORY,history);
+                pJson.put("history",history);
             }
         } else {
             // Single attribute, single bean. Value is the attribute_value
             // itself.
             addAttributeFromSingleValue(pJson,
-                                        KEY_HISTORY,
+                                        "history",
                                         new HistoryKey(pJmxReq),
-                                        pJson.get(KEY_VALUE),
+                                        pJson.get("value"),
                                         pTimestamp);
         }
     }
